@@ -49,6 +49,7 @@
 
 #include "conky.h"
 #include "display-output.hh"
+#include "geometry.h"
 #include "gui.h"
 #include "llua.h"
 #include "logging.h"
@@ -155,10 +156,10 @@ static int os_create_anonymous_file(off_t size) {
 #ifdef OWN_WINDOW
 extern int fixed_size, fixed_pos;
 #endif
-extern int text_start_x, text_start_y;   /* text start position in window */
-extern int text_offset_x, text_offset_y; /* offset for start position */
-extern int text_width,
-    text_height; /* initially 1 so no zero-sized window is created */
+extern conky::vec2i text_start;  /* text start position in window */
+extern conky::vec2i text_offset; /* offset for start position */
+extern conky::vec2i
+    text_size; /* initially 1 so no zero-sized window is created */
 extern double current_update_time, next_update_time, last_update_time;
 void update_text();
 extern int need_to_update;
@@ -259,12 +260,8 @@ static struct epoll_event ep[1];
 static struct window *global_window;
 static wl_display *global_display;
 
-struct rectangle {
-  size_t x, y, width, height;
-};
-
 struct window {
-  struct rectangle rectangle;
+  struct rect<size_t> rectangle;
   struct wl_shm *shm;
   struct wl_surface *surface;
   struct zwlr_layer_surface_v1 *layer_surface;
@@ -300,7 +297,17 @@ static void output_geometry(void *data, struct wl_output *wl_output, int32_t x,
                             int32_t y, int32_t physical_width,
                             int32_t physical_height, int32_t subpixel,
                             const char *make, const char *model,
-                            int32_t transform) {}
+                            int32_t transform) {
+  // TODO: Add support for proper output management through:
+  // - xdg-output-unstable-v1
+  // Maybe also support (if XDG protocol not reported):
+  // - kde-output-management(-v2)
+  // - wlr-output-management-unstable-v1
+  workarea.x = x;  // TODO: use xdg_output.logical_position
+  workarea.y = y;
+  workarea.width = physical_width;
+  workarea.height = physical_height;
+}
 
 static void output_mode(void *data, struct wl_output *wl_output, uint32_t flags,
                         int32_t width, int32_t height, int32_t refresh) {}
@@ -411,21 +418,19 @@ void window_layer_surface_set_size(struct window *window) {
 }
 
 #ifdef BUILD_MOUSE_EVENTS
-static std::map<wl_pointer *, std::array<size_t, 2>> last_known_positions{};
+static std::map<wl_pointer *, vec2<size_t>> last_known_positions{};
 
 static void on_pointer_enter(void *data, wl_pointer *pointer,
                              std::uint32_t serial, wl_surface *surface,
                              wl_fixed_t surface_x, wl_fixed_t surface_y) {
   auto w = reinterpret_cast<struct window *>(data);
 
-  size_t x = static_cast<size_t>(wl_fixed_to_double(surface_x));
-  size_t y = static_cast<size_t>(wl_fixed_to_double(surface_y));
-  last_known_positions[pointer] = std::array<size_t, 2>{x, y};
+  auto pos =
+      vec2d(wl_fixed_to_double(surface_x), wl_fixed_to_double(surface_y));
+  last_known_positions[pointer] = pos;
+  auto pos_abs = w->rectangle.pos + pos;
 
-  size_t abs_x = w->rectangle.x + x;
-  size_t abs_y = w->rectangle.y + y;
-
-  mouse_crossing_event event{mouse_event_t::AREA_ENTER, x, y, abs_x, abs_y};
+  mouse_crossing_event event{mouse_event_t::AREA_ENTER, pos, pos_abs};
   llua_mouse_hook(event);
 }
 
@@ -433,13 +438,10 @@ static void on_pointer_leave(void *data, struct wl_pointer *pointer,
                              std::uint32_t serial, struct wl_surface *surface) {
   auto w = reinterpret_cast<struct window *>(data);
 
-  std::array<size_t, 2> last = last_known_positions[pointer];
-  size_t x = last[0];
-  size_t y = last[1];
-  size_t abs_x = w->rectangle.x + x;
-  size_t abs_y = w->rectangle.y + y;
+  auto pos = last_known_positions[pointer];
+  auto pos_abs = w->rectangle.pos + pos;
 
-  mouse_crossing_event event{mouse_event_t::AREA_LEAVE, x, y, abs_x, abs_y};
+  mouse_crossing_event event{mouse_event_t::AREA_LEAVE, pos, pos_abs};
   llua_mouse_hook(event);
 }
 
@@ -448,14 +450,12 @@ static void on_pointer_motion(void *data, struct wl_pointer *pointer,
                               wl_fixed_t surface_y) {
   auto w = reinterpret_cast<struct window *>(data);
 
-  size_t x = static_cast<size_t>(wl_fixed_to_double(surface_x));
-  size_t y = static_cast<size_t>(wl_fixed_to_double(surface_y));
-  last_known_positions[pointer] = std::array<size_t, 2>{x, y};
+  auto pos =
+      vec2d(wl_fixed_to_double(surface_x), wl_fixed_to_double(surface_y));
+  last_known_positions[pointer] = pos;
+  auto pos_abs = w->rectangle.pos + pos;
 
-  size_t abs_x = w->rectangle.x + x;
-  size_t abs_y = w->rectangle.y + y;
-
-  mouse_move_event event{x, y, abs_x, abs_y};
+  mouse_move_event event{pos, pos_abs};
   llua_mouse_hook(event);
 }
 
@@ -464,18 +464,13 @@ static void on_pointer_button(void *data, struct wl_pointer *pointer,
                               std::uint32_t button, std::uint32_t state) {
   auto w = reinterpret_cast<struct window *>(data);
 
-  std::array<size_t, 2> last = last_known_positions[pointer];
-  size_t x = last[0];
-  size_t y = last[1];
-  size_t abs_x = w->rectangle.x + x;
-  size_t abs_y = w->rectangle.y + y;
+  auto pos = last_known_positions[pointer];
+  auto pos_abs = w->rectangle.pos + pos;
 
   mouse_button_event event{
       mouse_event_t::RELEASE,
-      x,
-      y,
-      abs_x,
-      abs_y,
+      pos,
+      pos_abs,
       static_cast<mouse_button_t>(button),
   };
 
@@ -498,14 +493,13 @@ void on_pointer_axis(void *data, struct wl_pointer *pointer, std::uint32_t time,
 
   auto w = reinterpret_cast<struct window *>(data);
 
-  std::array<size_t, 2> last = last_known_positions[pointer];
-  size_t x = last[0];
-  size_t y = last[1];
-  size_t abs_x = w->rectangle.x + x;
-  size_t abs_y = w->rectangle.y + y;
+  auto pos = last_known_positions[pointer];
+  auto pos_abs = w->rectangle.pos + pos;
 
   mouse_scroll_event event{
-      x, y, abs_x, abs_y, scroll_direction_t::UP,
+      pos,
+      pos_abs,
+      scroll_direction_t::UP,
   };
 
   switch (static_cast<wl_pointer_axis>(axis)) {
@@ -563,6 +557,7 @@ bool display_output_wayland::initialize() {
 
   wl_display_roundtrip(global_display);
   if (wl_globals.layer_shell == nullptr) {
+    // TODO: Implement OWN_WINDOW and XDG Shell support
     CRIT_ERR(
         "Compositor doesn't support wlr-layer-shell-unstable-v1. Can't run "
         "conky.");
@@ -662,25 +657,24 @@ bool display_output_wayland::main_loop_wait(double t) {
 
     /* resize window if it isn't right size */
     if ((fixed_size == 0) &&
-        (text_width + 2 * border_total != width ||
-         text_height + 2 * border_total != height || scale_changed)) {
+        (text_size.x + 2 * border_total != width ||
+         text_size.y + 2 * border_total != height || scale_changed)) {
       /* clamp text_width to configured maximum */
       if (maximum_width.get(*state)) {
         int mw = global_window->scale * maximum_width.get(*state);
-        if (text_width > mw && mw > 0) { text_width = mw; }
+        if (text_size.x > mw && mw > 0) { text_size.x = mw; }
       }
 
       /* pending scale will be applied by resizing the window */
       global_window->scale = global_window->pending_scale;
 
-      width = text_width + 2 * border_total;
-      height = text_height + 2 * border_total;
+      width = text_size.x + 2 * border_total;
+      height = text_size.y + 2 * border_total;
       window_resize(global_window, width, height); /* resize window */
 
       changed++;
       /* update lua window globals */
-      llua_update_window_table(text_start_x, text_start_y, text_width,
-                               text_height);
+      llua_update_window_table(conky::rect<int>(text_start, text_size));
     }
 
 /* move window if it isn't in right position */
@@ -814,8 +808,8 @@ int display_output_wayland::calc_text_width(const char *s) {
 }
 
 static void adjust_coords(int &x, int &y) {
-  x -= text_start_x;
-  y -= text_start_y;
+  x -= text_start.x;
+  y -= text_start.y;
   int border = get_border_total();
   x += border;
   y += border;
@@ -913,10 +907,7 @@ void display_output_wayland::move_win(int x, int y) {
   // window.y = y;
   // TODO
 }
-template <typename T, typename>
-T display_output_wayland::dpi_scale(T value) {
-  return value;
-}
+float display_output_wayland::get_dpi_scale() { return 1.0; }
 
 void display_output_wayland::end_draw_stuff() {
   window_commit_buffer(global_window);
@@ -1065,7 +1056,7 @@ static struct wl_shm_pool *make_shm_pool(struct wl_shm *shm, int size,
     return NULL;
   }
 
-  *data = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
+  *data = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_PRIVATE, fd, 0);
   if (*data == MAP_FAILED) {
     fprintf(stderr, "mmap failed: %m\n");
     close(fd);
@@ -1113,20 +1104,22 @@ static void shm_pool_destroy(struct shm_pool *pool) {
   delete pool;
 }
 
-static int stride_for_shm_surface(struct rectangle *rect, int scale) {
+static int stride_for_shm_surface(rect<size_t> *rect, int scale) {
   return cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32,
                                        rect->width * scale);
 }
 
-static int data_length_for_shm_surface(struct rectangle *rect, int scale) {
+static int data_length_for_shm_surface(rect<size_t> *rect, int scale) {
   int stride;
 
   stride = stride_for_shm_surface(rect, scale);
   return stride * rect->height * scale;
 }
 
-static cairo_surface_t *create_shm_surface_from_pool(
-    void *none, struct rectangle *rectangle, struct shm_pool *pool, int scale) {
+static cairo_surface_t *create_shm_surface_from_pool(void *none,
+                                                     rect<size_t> *rectangle,
+                                                     struct shm_pool *pool,
+                                                     int scale) {
   struct shm_surface_data *data;
   uint32_t format;
   cairo_surface_t *surface;
@@ -1149,18 +1142,18 @@ static cairo_surface_t *create_shm_surface_from_pool(
     return NULL;
   }
 
+  auto scaled = rectangle->size * scale;
   surface = cairo_image_surface_create_for_data(
-      static_cast<unsigned char *>(map), cairo_format, rectangle->width * scale,
-      rectangle->height * scale, stride);
+      static_cast<unsigned char *>(map), cairo_format, scaled.x, scaled.y,
+      stride);
 
   cairo_surface_set_user_data(surface, &shm_surface_data_key, data,
                               shm_surface_data_destroy);
 
   format = WL_SHM_FORMAT_ARGB8888; /*or WL_SHM_FORMAT_RGB565*/
 
-  data->buffer =
-      wl_shm_pool_create_buffer(pool->pool, offset, rectangle->width * scale,
-                                rectangle->height * scale, stride, format);
+  data->buffer = wl_shm_pool_create_buffer(pool->pool, offset, scaled.x,
+                                           scaled.y, stride, format);
 
   return surface;
 }
@@ -1202,10 +1195,8 @@ struct window *window_create(struct wl_surface *surface, struct wl_shm *shm,
   struct window *window;
   window = new struct window;
 
-  window->rectangle.x = 0;
-  window->rectangle.y = 0;
-  window->rectangle.width = width;
-  window->rectangle.height = height;
+  window->rectangle.pos = vec2<size_t>::Zero();
+  window->rectangle.size = vec2<size_t>(width, height);
   window->scale = 0;
   window->pending_scale = 1;
 
@@ -1244,8 +1235,7 @@ void window_destroy(struct window *window) {
 
 void window_resize(struct window *window, int width, int height) {
   window_free_buffer(window);
-  window->rectangle.width = width;
-  window->rectangle.height = height;
+  window->rectangle.size = conky::vec2i(width, height);
   window_allocate_buffer(window);
   window_layer_surface_set_size(window);
 }
