@@ -26,6 +26,8 @@
 
 #include "config.h"
 
+#include "display-x11.hh"
+
 #include <X11/X.h>
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wvariadic-macros"
@@ -49,6 +51,7 @@
 #include <X11/extensions/XInput2.h>
 #undef COUNT
 #endif /* BUILD_XINPUT */
+#include <X11/Xresource.h>
 
 #include <cstdint>
 #include <iostream>
@@ -60,30 +63,29 @@
 
 #include "colours.h"
 #include "conky.h"
-#include "display-x11.hh"
+#include "geometry.h"
 #include "gui.h"
 #include "llua.h"
-
 #include "logging.h"
+
+#include "x11-settings.h"
 #include "x11.h"
 
 // TODO: cleanup externs (move to conky.h ?)
 #ifdef OWN_WINDOW
 extern int fixed_size, fixed_pos;
-#endif                                   /* OWN_WINDOW */
-extern int text_start_x, text_start_y;   /* text start position in window */
-extern int text_offset_x, text_offset_y; /* offset for start position */
-extern int text_width,
-    text_height; /* initially 1 so no zero-sized window is created */
+#endif                           /* OWN_WINDOW */
+extern conky::vec2i text_start;  /* text start position in window */
+extern conky::vec2i text_offset; /* offset for start position */
+extern conky::vec2i
+    text_size; /* initially 1 so no zero-sized window is created */
 extern double current_update_time, next_update_time, last_update_time;
 void update_text();
 extern int need_to_update;
 int get_border_total();
 extern conky::range_config_setting<int> maximum_width;
 extern Colour current_color;
-#ifdef BUILD_XFT
-static int xft_dpi = -1;
-#endif /* BUILD_XFT */
+static float screen_dpi = -1;
 
 /* for x_fonts */
 struct x_font_list {
@@ -147,22 +149,42 @@ struct _x11_stuff_s {
 #endif
 } x11_stuff;
 
+void update_dpi() {
+  // Add XRandR support if used
+  // See dunst PR: https://github.com/dunst-project/dunst/pull/608
+
+#ifdef BUILD_XFT
+  if (screen_dpi > 0) return;
+  if (use_xft.get(*state)) {
+    XrmDatabase db = XrmGetDatabase(display);
+    if (db != nullptr) {
+      char *xrmType;
+      XrmValue xrmValue;
+      if (XrmGetResource(db, "Xft.dpi", "Xft.dpi", &xrmType, &xrmValue)) {
+        screen_dpi = strtof(xrmValue.addr, NULL);
+      }
+    } else {
+      auto dpi = XGetDefault(display, "Xft", "dpi");
+      if (dpi) { screen_dpi = strtof(dpi, nullptr); }
+    }
+  }
+#endif /* BUILD_XFT */
+  if (screen_dpi > 0) return;
+  screen_dpi = static_cast<float>(DisplayWidth(display, screen)) * 25.4 /
+               static_cast<float>(DisplayWidthMM(display, screen));
+}
+
 static void X11_create_window() {
   if (!window.window) { return; }
   setup_fonts();
   load_fonts(utf8_mode.get(*state));
-#ifdef BUILD_XFT
-  if (use_xft.get(*state)) {
-    auto dpi = XGetDefault(display, "Xft", "dpi");
-    if (dpi) { xft_dpi = strtol(dpi, nullptr, 10); }
-  }
-#endif                /* BUILD_XFT */
+  update_dpi();
   update_text_area(); /* to position text/window on screen */
 
 #ifdef OWN_WINDOW
   if (own_window.get(*state)) {
     if (fixed_pos == 0) {
-      XMoveWindow(display, window.window, window.x, window.y);
+      XMoveWindow(display, window.window, window.geometry.x, window.geometry.y);
     }
 
     set_transparent_background(window.window);
@@ -253,9 +275,10 @@ bool display_output_x11::main_loop_wait(double t) {
     }
   }
 
+  vec2i border_total = vec2i::uniform(get_border_total());
   if (need_to_update != 0) {
 #ifdef OWN_WINDOW
-    int wx = window.x, wy = window.y;
+    auto old_pos = window.geometry.pos;
 #endif
 
     need_to_update = 0;
@@ -265,17 +288,15 @@ bool display_output_x11::main_loop_wait(double t) {
 #ifdef OWN_WINDOW
     if (own_window.get(*state)) {
       int changed = 0;
-      int border_total = get_border_total();
 
       /* resize window if it isn't right size */
+      vec2<long> border_size = border_total * 2;
       if ((fixed_size == 0) &&
-          (text_width + 2 * border_total != window.width ||
-           text_height + 2 * border_total != window.height)) {
-        window.width = text_width + 2 * border_total;
-        window.height = text_height + 2 * border_total;
+          (text_size + border_size != window.geometry.size)) {
+        window.geometry.size = text_size + border_size;
         draw_stuff(); /* redraw everything in our newly sized window */
-        XResizeWindow(display, window.window, window.width,
-                      window.height); /* resize window */
+        XResizeWindow(display, window.window, window.geometry.width,
+                      window.geometry.height); /* resize window */
         set_transparent_background(window.window);
 #ifdef BUILD_XDBE
         /* swap buffers */
@@ -283,9 +304,9 @@ bool display_output_x11::main_loop_wait(double t) {
 #else
         if (use_xpmdb.get(*state)) {
           XFreePixmap(display, window.back_buffer);
-          window.back_buffer =
-              XCreatePixmap(display, window.window, window.width, window.height,
-                            DefaultDepth(display, screen));
+          window.back_buffer = XCreatePixmap(
+              display, window.window, window.geometry.width,
+              window.geometry.height, DefaultDepth(display, screen));
 
           if (window.back_buffer != None) {
             window.drawable = window.back_buffer;
@@ -295,19 +316,19 @@ bool display_output_x11::main_loop_wait(double t) {
           }
           XSetForeground(display, window.gc, 0);
           XFillRectangle(display, window.drawable, window.gc, 0, 0,
-                         window.width, window.height);
+                         window.geometry.width, window.geometry.height);
         }
 #endif
 
         changed++;
         /* update lua window globals */
-        llua_update_window_table(text_start_x, text_start_y, text_width,
-                                 text_height);
+        llua_update_window_table(rect<int>(text_start, text_size));
       }
 
       /* move window if it isn't in right position */
-      if ((fixed_pos == 0) && (window.x != wx || window.y != wy)) {
-        XMoveWindow(display, window.window, window.x, window.y);
+      if ((fixed_pos == 0) && old_pos != window.geometry.pos) {
+        XMoveWindow(display, window.window, window.geometry.x,
+                    window.geometry.y);
         changed++;
       }
 
@@ -326,14 +347,12 @@ bool display_output_x11::main_loop_wait(double t) {
 #else
     if (use_xpmdb.get(*state)) {
 #endif
-      XRectangle r;
-      int border_total = get_border_total();
+      conky::rect<int> r(text_start, text_size);
+      r.pos -= border_total;
+      r.size += border_total * 2;
 
-      r.x = text_start_x - border_total;
-      r.y = text_start_y - border_total;
-      r.width = text_width + 2 * border_total;
-      r.height = text_height + 2 * border_total;
-      XUnionRectWithRegion(&r, x11_stuff.region, x11_stuff.region);
+      XRectangle rect = r.to_xrectangle();
+      XUnionRectWithRegion(&rect, x11_stuff.region, x11_stuff.region);
     }
   }
 
@@ -359,14 +378,12 @@ bool display_output_x11::main_loop_wait(double t) {
 #else
     if (use_xpmdb.get(*state)) {
 #endif
-      XRectangle r;
-      int border_total = get_border_total();
+      conky::rect<int> r(text_start, text_size);
+      r.pos -= border_total;
+      r.size += border_total * 2;
 
-      r.x = text_start_x - border_total;
-      r.y = text_start_y - border_total;
-      r.width = text_width + 2 * border_total;
-      r.height = text_height + 2 * border_total;
-      XUnionRectWithRegion(&r, x11_stuff.region, x11_stuff.region);
+      XRectangle rect = r.to_xrectangle();
+      XUnionRectWithRegion(&rect, x11_stuff.region, x11_stuff.region);
     }
     XSetRegion(display, window.gc, x11_stuff.region);
 #ifdef BUILD_XFT
@@ -440,15 +457,12 @@ bool handle_event<x_event_handler::MOUSE_INPUT>(
   }
   *cookie = data;
 
-  Window event_window =
-      query_x11_window_at_pos(display, data->root_x, data->root_y);
+  Window event_window = query_x11_window_at_pos(display, data->pos_absolute);
 
   bool same_window = query_x11_top_parent(display, event_window) ==
                      query_x11_top_parent(display, window.window);
-  bool cursor_over_conky = same_window && data->root_x >= window.x &&
-                           data->root_x < (window.x + window.width) &&
-                           data->root_y >= window.y &&
-                           data->root_y < (window.y + window.height);
+  bool cursor_over_conky =
+      same_window && window.geometry.contains(data->pos_absolute);
 
   // XInput reports events twice on some hardware (even by 'xinput --test-xi2')
   auto hash = std::make_tuple(data->serial, data->evtype, data->event);
@@ -492,21 +506,21 @@ bool handle_event<x_event_handler::MOUSE_INPUT>(
       if (cursor_over_conky) {
         if (!cursor_inside) {
           *consumed = llua_mouse_hook(mouse_crossing_event(
-              mouse_event_t::AREA_ENTER, data->root_x - window.x,
-              data->root_y - window.x, data->root_x, data->root_y));
+              mouse_event_t::AREA_ENTER,
+              data->pos_absolute - window.geometry.pos, data->pos_absolute));
         }
         cursor_inside = true;
       } else if (cursor_inside) {
         *consumed = llua_mouse_hook(mouse_crossing_event(
-            mouse_event_t::AREA_LEAVE, data->root_x - window.x,
-            data->root_y - window.x, data->root_x, data->root_y));
+            mouse_event_t::AREA_LEAVE, data->pos_absolute - window.geometry.pos,
+            data->pos_absolute));
         cursor_inside = false;
       }
 
       // generate movement events
       if (cursor_over_conky) {
-        *consumed = llua_mouse_hook(mouse_move_event(
-            data->event_x, data->event_y, data->root_x, data->root_y, mods));
+        *consumed = llua_mouse_hook(
+            mouse_move_event(data->pos, data->pos_absolute, mods));
       }
     }
     if (is_scroll && cursor_over_conky) {
@@ -527,9 +541,8 @@ bool handle_event<x_event_handler::MOUSE_INPUT>(
       }
 
       if (scroll_direction != scroll_direction_t::UNKNOWN) {
-        *consumed = llua_mouse_hook(
-            mouse_scroll_event(data->event_x, data->event_y, data->root_x,
-                               data->root_y, scroll_direction, mods));
+        *consumed = llua_mouse_hook(mouse_scroll_event(
+            data->pos, data->pos_absolute, scroll_direction, mods));
       }
     }
   } else if (cursor_over_conky && (data->evtype == XI_ButtonPress ||
@@ -544,9 +557,8 @@ bool handle_event<x_event_handler::MOUSE_INPUT>(
     if (data->evtype == XI_ButtonRelease) { type = mouse_event_t::RELEASE; }
 
     mouse_button_t button = x11_mouse_button_code(data->detail);
-    *consumed = llua_mouse_hook(mouse_button_event(type, data->event_x,
-                                                   data->event_y, data->root_x,
-                                                   data->root_y, button, mods));
+    *consumed = llua_mouse_hook(
+        mouse_button_event(type, data->pos, data->pos_absolute, button, mods));
   }
 #endif /* BUILD_MOUSE_EVENTS */
 
@@ -636,10 +648,10 @@ bool handle_event<x_event_handler::CONFIGURE>(
   if (ev.type != ConfigureNotify) return false;
 
   if (own_window.get(*state)) {
-    /* if window size isn't what expected, set fixed size */
-    if (ev.xconfigure.width != window.width ||
-        ev.xconfigure.height != window.height) {
-      if (window.width != 0 && window.height != 0) { fixed_size = 1; }
+    auto configure_size = vec2i(ev.xconfigure.width, ev.xconfigure.height);
+    /* if window size isn't what's expected, set fixed size */
+    if (configure_size != window.geometry.size) {
+      if (window.geometry.size.surface() != 0) { fixed_size = 1; }
 
       /* clear old stuff before screwing up
        * size and pos */
@@ -648,17 +660,18 @@ bool handle_event<x_event_handler::CONFIGURE>(
       {
         XWindowAttributes attrs;
         if (XGetWindowAttributes(display, window.window, &attrs) != 0) {
-          window.width = attrs.width;
-          window.height = attrs.height;
+          window.geometry.size = vec2i(attrs.width, attrs.height);
         }
       }
 
-      int border_total = get_border_total();
+      auto border_total = vec2i::uniform(get_border_total() * 2);
+      auto new_text_size = window.geometry.size - border_total;
 
-      text_width = window.width - 2 * border_total;
-      text_height = window.height - 2 * border_total;
-      int mw = surface->dpi_scale(maximum_width.get(*state));
-      if (text_width > mw && mw > 0) { text_width = mw; }
+      text_size = new_text_size;
+
+      // don't apply dpi scaling to max pixel size
+      int mw = maximum_width.get(*state);
+      if (text_size.x > mw && mw > 0) { text_size.x = mw; }
     }
 
     /* if position isn't what expected, set fixed pos
@@ -666,8 +679,8 @@ bool handle_event<x_event_handler::CONFIGURE>(
      * is set to weird locations when started */
     /* // this is broken
     if (total_updates >= 2 && !fixed_pos
-        && (window.x != ev.xconfigure.x
-        || window.y != ev.xconfigure.y)
+        && (window.geometry.x != ev.xconfigure.x
+        || window.geometry.y != ev.xconfigure.y)
         && (ev.xconfigure.x != 0
         || ev.xconfigure.y != 0)) {
       fixed_pos = 1;
@@ -685,18 +698,16 @@ bool handle_event<x_event_handler::BORDER_CROSSING>(
   if (ev.type != EnterNotify && ev.type != LeaveNotify) return false;
   if (window.xi_opcode != 0) return true;  // handled by mouse_input already
 
-  bool not_over_conky = ev.xcrossing.x_root <= window.x ||
-                        ev.xcrossing.y_root <= window.y ||
-                        ev.xcrossing.x_root >= window.x + window.width ||
-                        ev.xcrossing.y_root >= window.y + window.height;
+  auto crossing_pos = vec2i(ev.xcrossing.x_root, ev.xcrossing.y_root);
+  bool over_conky = window.geometry.contains(crossing_pos);
 
-  if ((not_over_conky && ev.xcrossing.type == LeaveNotify) ||
-      (!not_over_conky && ev.xcrossing.type == EnterNotify)) {
+  if ((!over_conky && ev.xcrossing.type == LeaveNotify) ||
+      (over_conky && ev.xcrossing.type == EnterNotify)) {
     llua_mouse_hook(mouse_crossing_event(
         ev.xcrossing.type == EnterNotify ? mouse_event_t::AREA_ENTER
                                          : mouse_event_t::AREA_LEAVE,
-        ev.xcrossing.x, ev.xcrossing.y, ev.xcrossing.x_root,
-        ev.xcrossing.y_root));
+        vec2i(ev.xcrossing.x, ev.xcrossing.y),
+        vec2i(ev.xcrossing.x_root, ev.xcrossing.y_root)));
   }
   return true;
 }
@@ -713,19 +724,31 @@ bool handle_event<x_event_handler::PROPERTY_NOTIFY>(
     get_x11_desktop_info(ev.xproperty.display, ev.xproperty.atom);
   }
 
-#ifdef USE_ARGB
-  if (have_argb_visual) return true;
-#endif
+  if (ev.xproperty.atom == 0) return false;
 
-  if (ev.xproperty.atom == ATOM(_XROOTPMAP_ID) ||
-      ev.xproperty.atom == ATOM(_XROOTMAP_ID)) {
-    if (forced_redraw.get(*state)) {
-      draw_stuff();
-      next_update_time = get_time();
-      need_to_update = 1;
+  if (ev.xproperty.atom == XA_RESOURCE_MANAGER) {
+    update_x11_resource_db();
+    update_x11_workarea();
+    screen_dpi = -1;
+    update_dpi();
+    return true;
+  }
+
+  if (!have_argb_visual) {
+    Atom _XROOTPMAP_ID = XInternAtom(display, "_XROOTPMAP_ID", True);
+    Atom _XROOTMAP_ID = XInternAtom(display, "_XROOTMAP_ID", True);
+    if (ev.xproperty.atom == _XROOTPMAP_ID ||
+        ev.xproperty.atom == _XROOTMAP_ID) {
+      if (forced_redraw.get(*state)) {
+        draw_stuff();
+        next_update_time = get_time();
+        need_to_update = 1;
+      }
+      return true;
     }
   }
-  return true;
+
+  return false;
 }
 
 template <>
@@ -734,11 +757,12 @@ bool handle_event<x_event_handler::EXPOSE>(conky::display_output_x11 *surface,
                                            bool *consumed, void **cookie) {
   if (ev.type != Expose) return false;
 
-  XRectangle r;
-  r.x = ev.xexpose.x;
-  r.y = ev.xexpose.y;
-  r.width = ev.xexpose.width;
-  r.height = ev.xexpose.height;
+  XRectangle r{
+      .x = static_cast<short>(ev.xexpose.x),
+      .y = static_cast<short>(ev.xexpose.y),
+      .width = static_cast<unsigned short>(ev.xexpose.width),
+      .height = static_cast<unsigned short>(ev.xexpose.height),
+  };
   XUnionRectWithRegion(&r, x11_stuff.region, x11_stuff.region);
   XSync(display, False);
   return true;
@@ -835,9 +859,9 @@ void display_output_x11::cleanup() {
   if (window_created == 1) {
     int border_total = get_border_total();
 
-    XClearArea(display, window.window, text_start_x - border_total,
-               text_start_y - border_total, text_width + 2 * border_total,
-               text_height + 2 * border_total, 0);
+    XClearArea(display, window.window, text_start.x - border_total,
+               text_start.y - border_total, text_size.x + 2 * border_total,
+               text_size.y + 2 * border_total, 0);
   }
   destroy_window();
   free_fonts(utf8_mode.get(*state));
@@ -939,24 +963,17 @@ void display_output_x11::draw_arc(int x, int y, int w, int h, int a1, int a2) {
 
 void display_output_x11::move_win(int x, int y) {
 #ifdef OWN_WINDOW
-  window.x = x;
-  window.y = y;
+  window.geometry.pos = vec2i(x, y);
   XMoveWindow(display, window.window, x, y);
 #endif /* OWN_WINDOW */
 }
 
-const size_t PIXELS_PER_INCH = 96;
-template <typename T, typename>
-T display_output_x11::dpi_scale(T value) {
-#if defined(BUILD_XFT)
-  if (use_xft.get(*state) && xft_dpi > 0) {
-    return (value * xft_dpi + (value > 0 ? 48 : -48)) / PIXELS_PER_INCH;
-  } else {
-    return value;
+const float PIXELS_PER_INCH = 96.0;
+float display_output_x11::get_dpi_scale() {
+  if (screen_dpi > 0) {
+    return static_cast<float>(screen_dpi) / PIXELS_PER_INCH;
   }
-#else  /* defined(BUILD_XFT) */
-  return value;
-#endif /* defined(BUILD_XFT) */
+  return 1.0;
 }
 
 void display_output_x11::end_draw_stuff() {
@@ -983,9 +1000,9 @@ void display_output_x11::clear_text(int exposures) {
     /* there is some extra space for borders and outlines */
     int border_total = get_border_total();
 
-    XClearArea(display, window.window, text_start_x - border_total,
-               text_start_y - border_total, text_width + 2 * border_total,
-               text_height + 2 * border_total, exposures != 0 ? True : 0);
+    XClearArea(display, window.window, text_start.x - border_total,
+               text_start.y - border_total, text_size.x + 2 * border_total,
+               text_size.y + 2 * border_total, exposures != 0 ? True : 0);
   }
 }
 
