@@ -27,13 +27,17 @@
  *
  */
 
+#include "config.h"
+
+#include "x11-settings.h"
 #include "x11.h"
 
 #include <X11/X.h>
 #include <sys/types.h>
+
 #include "common.h"
-#include "config.h"
 #include "conky.h"
+#include "geometry.h"
 #include "gui.h"
 #include "logging.h"
 
@@ -90,10 +94,11 @@ extern "C" {
 #include <xcb/xcb.h>
 #include <xcb/xcb_errors.h>
 #endif
+#include <X11/Xresource.h>
 }
 
-/* some basic X11 stuff */
 Display *display = nullptr;
+int screen;
 
 #ifdef HAVE_XCB_ERRORS
 xcb_connection_t *xcb_connection;
@@ -106,9 +111,8 @@ struct conky_x11_window window;
 bool have_argb_visual = false;
 
 /* local prototypes */
-static void update_workarea();
 static Window find_desktop_window(Window *p_root, Window *p_desktop);
-static Window find_subwindow(Window win, int w, int h);
+static Window find_desktop_window_impl(Window win, int w, int h);
 
 /* WARNING, this type not in Xlib spec */
 static int x11_error_handler(Display *d, XErrorEvent *err) {
@@ -192,36 +196,35 @@ __attribute__((noreturn)) static int x11_ioerror_handler(Display *d) {
 /// manage workspaces. These are direct descendants of root and WMs reparent all
 /// children to them.
 ///
-/// @param screen screen to get the (current) virtual root of @return the
-/// virtual root window of the screen
+/// @param screen screen to get the (current) virtual root of
+/// @return the virtual root window of the screen
 static Window VRootWindowOfScreen(Screen *screen) {
-  Window root = screen->root;
-  Display *dpy = screen->display;
+  Window root = RootWindowOfScreen(screen);
+  Display *dpy = DisplayOfScreen(screen);
+
+  /* go look for a virtual root */
+  Atom _NET_VIRTUAL_ROOTS = XInternAtom(display, "_NET_VIRTUAL_ROOTS", True);
+  if (_NET_VIRTUAL_ROOTS == 0) return root;
+
+  auto vroots = x11_atom_window_list(dpy, root, _NET_VIRTUAL_ROOTS);
+
+  if (vroots.empty()) return root;
+
+  Atom _NET_CURRENT_DESKTOP =
+      XInternAtom(display, "_NET_CURRENT_DESKTOP", True);
+  if (_NET_CURRENT_DESKTOP == 0) return root;
 
-  Window rootReturn, parentReturn, *children;
-  unsigned int numChildren;
   Atom actual_type;
   int actual_format;
   unsigned long nitems, bytesafter;
+  int *cardinal;
 
-  /* go look for a virtual root */
-  Atom __SWM_VROOT = ATOM(__SWM_VROOT);
-  if (XQueryTree(dpy, root, &rootReturn, &parentReturn, &children,
-                 &numChildren)) {
-    for (int i = 0; i < numChildren; i++) {
-      Window *newRoot = None;
+  XGetWindowProperty(dpy, root, _NET_CURRENT_DESKTOP, 0, 1, False, XA_CARDINAL,
+                     &actual_type, &actual_format, &nitems, &bytesafter,
+                     (unsigned char **)&cardinal);
 
-      if (XGetWindowProperty(
-              dpy, children[i], __SWM_VROOT, 0, 1, False, XA_WINDOW,
-              &actual_type, &actual_format, &nitems, &bytesafter,
-              reinterpret_cast<unsigned char **>(&newRoot)) == Success &&
-          newRoot != None) {
-        root = *newRoot;
-        break;
-      }
-    }
-    if (children) XFree((char *)children);
-  }
+  if (vroots.size() > *cardinal) { root = vroots[*cardinal]; }
+  XFree(cardinal);
 
   return root;
 }
@@ -261,26 +264,23 @@ void init_x11() {
   info.x11.desktop.name.clear();
 
   screen = DefaultScreen(display);
-  display_width = DisplayWidth(display, screen);
-  display_height = DisplayHeight(display, screen);
+
+  XSetErrorHandler(&x11_error_handler);
+  XSetIOErrorHandler(&x11_ioerror_handler);
+
+  update_x11_resource_db(true);
+  update_x11_workarea();
 
   get_x11_desktop_info(display, 0);
 
-  update_workarea();
-
 #ifdef HAVE_XCB_ERRORS
   auto connection = xcb_connect(NULL, NULL);
   if (!xcb_connection_has_error(connection)) {
-    if (xcb_errors_context_new(connection, &xcb_errors_ctx) != Success) {
+    if (xcb_errors_context_new(connection, &xcb_errors_ctx) != 0) {
       xcb_errors_ctx = nullptr;
     }
   }
 #endif /* HAVE_XCB_ERRORS */
-
-  /* WARNING, this type not in Xlib spec */
-  XSetErrorHandler(&x11_error_handler);
-  XSetIOErrorHandler(&x11_ioerror_handler);
-
   DBGP("leave init_x11()");
 }
 
@@ -292,12 +292,38 @@ void deinit_x11() {
   }
 }
 
-static void update_workarea() {
+// Source: dunst
+// https://github.com/bebehei/dunst/blob/1bc3237a359f37905426012c0cca90d71c4b3b18/src/x11/x.c#L463
+void update_x11_resource_db(bool first_run) {
+  XrmDatabase db;
+  XTextProperty prop;
+  Window root;
+
+  XFlush(display);
+
+  root = RootWindow(display, screen);
+
+  XLockDisplay(display);
+  if (XGetTextProperty(display, root, &prop, XA_RESOURCE_MANAGER)) {
+    if (!first_run) {
+      db = XrmGetDatabase(display);
+      XrmDestroyDatabase(db);
+    }
+
+    db = XrmGetStringDatabase((const char *)prop.value);
+    XrmSetDatabase(display, db);
+  }
+  XUnlockDisplay(display);
+
+  XFlush(display);
+  XSync(display, false);
+}
+
+void update_x11_workarea() {
   /* default work area is display */
-  workarea[0] = 0;
-  workarea[1] = 0;
-  workarea[2] = display_width;
-  workarea[3] = display_height;
+  workarea.pos = conky::vec2i();
+  workarea.width = DisplayWidth(display, screen);
+  workarea.height = DisplayHeight(display, screen);
 
 #ifdef BUILD_XINERAMA
   /* if xinerama is being used, adjust workarea to the head's area */
@@ -345,9 +371,12 @@ static Window find_desktop_window(Window root) {
   Window desktop = root;
 
   /* get subwindows from root */
-  desktop = find_subwindow(root, -1, -1);
-  update_workarea();
-  desktop = find_subwindow(desktop, workarea[2], workarea[3]);
+  int display_width = DisplayWidth(display, screen);
+  int display_height = DisplayHeight(display, screen);
+  desktop = find_desktop_window_impl(root, display_width, display_height);
+  update_x11_workarea();
+  desktop = find_desktop_window_impl(desktop, workarea[2] - workarea[0],
+                                     workarea[3] - workarea[1]);
 
   if (desktop != root) {
     NORM_ERR("desktop window (0x%lx) is subwindow of root window (0x%lx)",
@@ -515,18 +544,16 @@ void x11_init_window(lua::state &l, bool own) {
                                     0,
                                     0};
       flags |= CWBackPixel;
-#ifdef BUILD_ARGB
       if (have_argb_visual) {
         attrs.colormap = window.colourmap;
         flags &= ~CWBackPixel;
         flags |= CWBorderPixel | CWColormap;
       }
-#endif /* BUILD_ARGB */
 
       /* Parent is desktop window (which might be a child of root) */
-      window.window =
-          XCreateWindow(display, window.desktop, window.x, window.y, b, b, 0,
-                        depth, InputOutput, visual, flags, &attrs);
+      window.window = XCreateWindow(display, window.desktop, window.geometry.x,
+                                    window.geometry.y, b, b, 0, depth,
+                                    InputOutput, visual, flags, &attrs);
 
       XLowerWindow(display, window.window);
       XSetClassHint(display, window.window, &classHint);
@@ -558,21 +585,19 @@ void x11_init_window(lua::state &l, bool own) {
       Atom xa;
 
       flags |= CWBackPixel;
-#ifdef BUILD_ARGB
       if (have_argb_visual) {
         attrs.colormap = window.colourmap;
         flags &= ~CWBackPixel;
         flags |= CWBorderPixel | CWColormap;
       }
-#endif /* BUILD_ARGB */
 
       if (own_window_type.get(l) == window_type::DOCK) {
-        window.x = window.y = 0;
+        window.geometry.pos = conky::vec2i::Zero();
       }
       /* Parent is root window so WM can take control */
-      window.window =
-          XCreateWindow(display, window.root, window.x, window.y, b, b, 0,
-                        depth, InputOutput, visual, flags, &attrs);
+      window.window = XCreateWindow(display, window.root, window.geometry.x,
+                                    window.geometry.y, b, b, 0, depth,
+                                    InputOutput, visual, flags, &attrs);
 
       uint16_t hints = own_window_hints.get(l);
 
@@ -768,8 +793,7 @@ void x11_init_window(lua::state &l, bool own) {
     if (window.window == None) { window.window = window.desktop; }
 
     if (XGetWindowAttributes(display, window.window, &attrs) != 0) {
-      window.width = attrs.width;
-      window.height = attrs.height;
+      window.geometry.size = conky::vec2i(attrs.width, attrs.height);
     }
 
     NORM_ERR("drawing to desktop window");
@@ -803,7 +827,7 @@ void x11_init_window(lua::state &l, bool own) {
 
     int major = 2, minor = 0;
     int retval = XIQueryVersion(display, &major, &minor);
-    if (retval != Success) {
+    if (retval != 0) {
       NORM_ERR("Error: XInput 2.0 is not supported!");
       break;
     }
@@ -869,7 +893,7 @@ void x11_init_window(lua::state &l, bool own) {
   DBGP("leave x11_init_window()");
 }
 
-static Window find_subwindow(Window win, int w, int h) {
+static Window find_desktop_window_impl(Window win, int w, int h) {
   unsigned int i, j;
   Window troot, parent, *children;
   unsigned int n;
@@ -881,13 +905,11 @@ static Window find_subwindow(Window win, int w, int h) {
 
     for (j = 0; j < n; j++) {
       XWindowAttributes attrs;
-
       if (XGetWindowAttributes(display, children[j], &attrs) != 0) {
         /* Window must be mapped and same size as display or
          * work space */
-        if (attrs.map_state != 0 &&
-            ((attrs.width == display_width && attrs.height == display_height) ||
-             (attrs.width == w && attrs.height == h))) {
+        if (attrs.map_state == IsViewable && attrs.override_redirect == false &&
+            ((attrs.width == w && attrs.height == h))) {
           win = children[j];
           break;
         }
@@ -924,7 +946,7 @@ static inline void get_x11_desktop_current(Display *current_display,
 
   if ((XGetWindowProperty(current_display, root, atom, 0, 1L, False,
                           XA_CARDINAL, &actual_type, &actual_format, &nitems,
-                          &bytes_after, &prop) == Success) &&
+                          &bytes_after, &prop) == 0) &&
       (actual_type == XA_CARDINAL) && (nitems == 1L) && (actual_format == 32)) {
     current_info->x11.desktop.current = prop[0] + 1;
   }
@@ -945,7 +967,7 @@ static inline void get_x11_desktop_number(Display *current_display, Window root,
 
   if ((XGetWindowProperty(current_display, root, atom, 0, 1L, False,
                           XA_CARDINAL, &actual_type, &actual_format, &nitems,
-                          &bytes_after, &prop) == Success) &&
+                          &bytes_after, &prop) == 0) &&
       (actual_type == XA_CARDINAL) && (nitems == 1L) && (actual_format == 32)) {
     current_info->x11.desktop.number = prop[0];
   }
@@ -966,7 +988,7 @@ static inline void get_x11_desktop_names(Display *current_display, Window root,
 
   if ((XGetWindowProperty(current_display, root, atom, 0, (~0L), False,
                           ATOM(UTF8_STRING), &actual_type, &actual_format,
-                          &nitems, &bytes_after, &prop) == Success) &&
+                          &nitems, &bytes_after, &prop) == 0) &&
       (actual_type == ATOM(UTF8_STRING)) && (nitems > 0L) &&
       (actual_format == 8)) {
     current_info->x11.desktop.all_names.assign(
@@ -1117,40 +1139,44 @@ void set_struts(alignment align) {
   if (strut != None) {
     long sizes[STRUT_COUNT] = {0};
 
+    int display_width = workarea[2] - workarea[0];
+    int display_height = workarea[3] - workarea[1];
+
     switch (horizontal_alignment(align)) {
       case axis_align::START:
-        sizes[*x11_strut::LEFT] =
-            std::clamp(window.x + window.width, 0, display_width);
+        sizes[*x11_strut::LEFT] = std::clamp(
+            window.geometry.x + window.geometry.width, 0, *workarea.width);
         sizes[*x11_strut::LEFT_START_Y] =
-            std::clamp(window.y, 0, display_height);
-        sizes[*x11_strut::LEFT_END_Y] =
-            std::clamp(window.y + window.height, 0, display_height);
+            std::clamp(*window.geometry.y, 0, *workarea.height);
+        sizes[*x11_strut::LEFT_END_Y] = std::clamp(
+            window.geometry.y + window.geometry.height, 0, *workarea.height);
         break;
       case axis_align::END:
         sizes[*x11_strut::RIGHT] =
-            std::clamp(display_width - window.x, 0, display_width);
+            std::clamp(workarea.width - window.geometry.x, 0, *workarea.width);
         sizes[*x11_strut::RIGHT_START_Y] =
-            std::clamp(window.y, 0, display_height);
-        sizes[*x11_strut::RIGHT_END_Y] =
-            std::clamp(window.y + window.height, 0, display_height);
+            std::clamp(*window.geometry.y, 0, *workarea.height);
+        sizes[*x11_strut::RIGHT_END_Y] = std::clamp(
+            window.geometry.y + window.geometry.height, 0, *workarea.height);
         break;
       case axis_align::MIDDLE:
         switch (vertical_alignment(align)) {
           case axis_align::START:
             sizes[*x11_strut::TOP] =
-                std::clamp(window.y + window.height, 0, display_height);
+                std::clamp(window.geometry.y + window.geometry.height, 0,
+                           *workarea.height);
             sizes[*x11_strut::TOP_START_X] =
-                std::clamp(window.x, 0, display_width);
-            sizes[*x11_strut::TOP_END_X] =
-                std::clamp(window.x + window.width, 0, display_width);
+                std::clamp(*window.geometry.x, 0, *workarea.width);
+            sizes[*x11_strut::TOP_END_X] = std::clamp(
+                window.geometry.x + window.geometry.width, 0, *workarea.width);
             break;
           case axis_align::END:
-            sizes[*x11_strut::BOTTOM] =
-                std::clamp(display_height - window.y, 0, display_height);
+            sizes[*x11_strut::BOTTOM] = std::clamp(
+                workarea.height - window.geometry.y, 0, *workarea.height);
             sizes[*x11_strut::BOTTOM_START_X] =
-                std::clamp(window.x, 0, display_width);
-            sizes[*x11_strut::BOTTOM_END_X] =
-                std::clamp(window.x + window.width, 0, display_width);
+                std::clamp(*window.geometry.x, 0, *workarea.width);
+            sizes[*x11_strut::BOTTOM_END_X] = std::clamp(
+                window.geometry.x + window.geometry.width, 0, *workarea.width);
             break;
           case axis_align::MIDDLE:
             // can't reserve space in middle of the screen
@@ -1295,11 +1321,10 @@ void propagate_xinput_event(const conky::xi_event_data *ev) {
 
   Window target = window.root;
   Window child = None;
-  int target_x = ev->event_x;
-  int target_y = ev->event_y;
+  conky::vec2i target_pos = ev->pos;
   {
     std::vector<Window> below = query_x11_windows_at_pos(
-        display, ev->root_x, ev->root_y,
+        display, ev->pos_absolute,
         [](XWindowAttributes &a) { return a.map_state == IsViewable; });
     auto it = std::remove_if(below.begin(), below.end(),
                              [](Window w) { return w == window.window; });
@@ -1307,13 +1332,16 @@ void propagate_xinput_event(const conky::xi_event_data *ev) {
     if (!below.empty()) {
       target = below.back();
 
+      int read_x, read_y;
       // Update event x and y coordinates to be target window relative
-      XTranslateCoordinates(display, window.desktop, ev->event, ev->root_x,
-                            ev->root_y, &target_x, &target_y, &child);
+      XTranslateCoordinates(display, window.desktop, ev->event,
+                            ev->pos_absolute.x, ev->pos_absolute.y, &read_x,
+                            &read_y, &child);
+      target_pos = conky::vec2i(read_x, read_y);
     }
   }
 
-  auto events = ev->generate_events(target, child, target_x, target_y);
+  auto events = ev->generate_events(target, child, target_pos);
 
   XUngrabPointer(display, CurrentTime);
   for (auto it : events) {
@@ -1358,7 +1386,7 @@ void propagate_x11_event(XEvent &ev, const void *cookie) {
   /* forward the event to the window below conky (e.g. caja) or desktop */
   {
     std::vector<Window> below = query_x11_windows_at_pos(
-        display, ev.xbutton.x_root, ev.xbutton.y_root,
+        display, conky::vec2i(ev.xbutton.x_root, ev.xbutton.y_root),
         [](XWindowAttributes &a) { return a.map_state == IsViewable; });
     auto it = std::remove_if(below.begin(), below.end(),
                              [](Window w) { return w == window.window; });
@@ -1417,7 +1445,7 @@ std::vector<Window> x11_atom_window_list(Display *display, Window window,
 
   if (XGetWindowProperty(display, window, atom, 0, (~0L), False, XA_WINDOW,
                          &actual_type, &actual_format, &nitems, &bytes_after,
-                         &data) == Success) {
+                         &data) == 0) {
     if (actual_format == XA_WINDOW && nitems > 0) {
       Window *wdata = reinterpret_cast<Window *>(data);
       std::vector<Window> result(wdata, wdata + nitems);
@@ -1429,46 +1457,54 @@ std::vector<Window> x11_atom_window_list(Display *display, Window window,
   return std::vector<Window>{};
 }
 
-std::vector<Window> query_x11_windows(Display *display) {
+std::vector<Window> query_x11_windows(Display *display, bool eager) {
   Window root = DefaultRootWindow(display);
 
-  Atom clients_atom = ATOM(_NET_CLIENT_LIST_STACKING);
-  std::vector<Window> result =
-      x11_atom_window_list(display, root, clients_atom);
-  if (result.empty()) { return result; }
+  std::vector<Window> result;
 
-  clients_atom = ATOM(_NET_CLIENT_LIST);
-  result = x11_atom_window_list(display, root, clients_atom);
-  if (result.empty()) { return result; }
+  Atom clients_atom = XInternAtom(display, "_NET_CLIENT_LIST_STACKING", True);
+  if (clients_atom != 0) {
+    result = x11_atom_window_list(display, root, clients_atom);
+    if (!result.empty()) { return result; }
+  }
+
+  clients_atom = XInternAtom(display, "_NET_CLIENT_LIST", True);
+  if (clients_atom != 0) {
+    result = x11_atom_window_list(display, root, clients_atom);
+    if (!result.empty()) { return result; }
+  }
 
   // slowest method
 
-  std::vector<Window> queue = {DefaultVRootWindow(display)};
+  if (eager) {
+    std::vector<Window> queue = {DefaultVRootWindow(display)};
 
-  Window _ignored, *children;
-  std::uint32_t count;
+    Window _ignored, *children;
+    std::uint32_t count;
 
-  const auto has_wm_hints = [&](Window window) {
-    auto hints = XGetWMHints(display, window);
-    bool result = hints != NULL;
-    if (result) XFree(hints);
-    return result;
-  };
+    const auto has_wm_hints = [&](Window window) {
+      auto hints = XGetWMHints(display, window);
+      bool result = hints != NULL;
+      if (result) XFree(hints);
+      return result;
+    };
 
-  while (!queue.empty()) {
-    Window current = queue.back();
-    queue.pop_back();
-    if (XQueryTree(display, current, &_ignored, &_ignored, &children, &count)) {
-      for (size_t i = 0; i < count; i++) queue.push_back(children[i]);
-      if (has_wm_hints(current)) result.push_back(current);
-      if (count > 0) XFree(children);
+    while (!queue.empty()) {
+      Window current = queue.back();
+      queue.pop_back();
+      if (XQueryTree(display, current, &_ignored, &_ignored, &children,
+                     &count)) {
+        for (size_t i = 0; i < count; i++) queue.push_back(children[i]);
+        if (has_wm_hints(current)) result.push_back(current);
+        if (count > 0) XFree(children);
+      }
     }
   }
 
   return result;
 }
 
-Window query_x11_window_at_pos(Display *display, int x, int y) {
+Window query_x11_window_at_pos(Display *display, conky::vec2i pos) {
   Window root = DefaultVRootWindow(display);
 
   // these values are ignored but NULL can't be passed to XQueryPointer.
@@ -1485,14 +1521,14 @@ Window query_x11_window_at_pos(Display *display, int x, int y) {
 }
 
 std::vector<Window> query_x11_windows_at_pos(
-    Display *display, int x, int y,
-    std::function<bool(XWindowAttributes &)> predicate) {
+    Display *display, conky::vec2i pos,
+    std::function<bool(XWindowAttributes &)> predicate, bool eager) {
   std::vector<Window> result;
 
   Window root = DefaultVRootWindow(display);
   XWindowAttributes attr;
 
-  for (Window current : query_x11_windows(display)) {
+  for (Window current : query_x11_windows(display, eager)) {
     int pos_x, pos_y;
     Window _ignore;
     // Doesn't account for decorations. There's no sane way to do that.
@@ -1500,8 +1536,8 @@ std::vector<Window> query_x11_windows_at_pos(
                           &_ignore);
     XGetWindowAttributes(display, current, &attr);
 
-    if (pos_x <= x && pos_y <= y && pos_x + attr.width >= x &&
-        pos_y + attr.height >= y && predicate(attr)) {
+    if (pos_x <= pos.x && pos_y <= pos.y && pos_x + attr.width >= pos.x &&
+        pos_y + attr.height >= pos.y && predicate(attr)) {
       result.push_back(current);
     }
   }
